import socket
import atom
import gdata
from gdata import service;
import gdata.photos
import gdata.photos.service
import re
import os

import gconf

import gettext
_ = gettext.gettext

from gnomeblog import hig_alert
from gnomeblog import proxy

class Blog:
    def __init__(self):
        pass
    
    def _getURL(self, base_url, client, gconf_prefix):
        url_ending = client.get_string (gconf_prefix + "/url_ending")
        return base_url + url_ending
    
    def getBlogList(self, username, password, base_url, client, gconf_prefix):
        # TODO: consider GNOME proxy configurations, as in bloggerAPI.py
        error_name = _("Could not get list of blogs.")
        
        try:
            blogger_service = self._login (username, password)
        except service.CaptchaRequired as message:
            hig_alert.reportError (error_name, _("A captcha was required for authentication.")) # TODO: propagadate captcha
            return None
        except service.BadAuthentication as message:
            hig_alert.reportError (error_name, _("Username or password was invalid."))
            return None
        except service.Error as message:
            hig_alert.reportError (error_name, _("Failed to login: %s" % message))
            return None
        except socket.gaierror as message:
            hig_alert.reportError (error_name, _("Socket error: %s.  Perhaps try again." % message));
            return None

        blog_list = blogger_service.Get ('/feeds/default/blogs'); # is there anyway to sort these by recently used?

        if ((blog_list.entry == None) or (len(blog_list.entry) == 0)):
            # No blogs found
            hig_alert.reportError (_("No Blogs Found"), _("No errors were reported, but no blogs were found at %s for %s\n" % (url, username)));

        string_value_pairs = []

        for blog in blog_list.entry:
            blog_title = blog.title.text
            blog_id = blog.id.text.split ('-')[-1]
            string_value_pairs.append ( (blog_title, blog_id) );

        return string_value_pairs

    def _login (self, username, password):
        if username.endswith ('@gmail.com') == False: # can use blogger with a non-Gmail address? 
            username = username + "@gmail.com";

        service = gdata.service.GDataService (username, password)
        service.source = "Gnome Blog"
        service.service = "blogger"
        service.server = "www.blogger.com"
        service.ProgrammaticLogin ()

        return service
    
    def postEntry (self, username, password, base_url, title, entry, keywords, client, gconf_prefix):
        blog_id = client.get_string (gconf_prefix + "/blog_id");

        if (blog_id == None):
            blog_id = ""

        if (username == None):
            username = ""

        if (password == None):
            password = ""

        url = self._getURL(base_url, client, gconf_prefix)

        # consider GNOME proxy configuration, but perhaps python's gdata lib does that? 

        post = gdata.GDataEntry ()
        post.title = atom.Title (title_type = 'xhtml', text = title)
        post.content = atom.Content (content_type = 'html', text = entry)
        
        if keywords is not None and keywords != "":
            scheme = "http://www.blogger.com/atom/ns#"
            for keyword in keywords.split (","):
                post.category.append (atom.Category (keyword, scheme))
            
        success = True

        error_name = _("Could not post entry.")
        try:
            blogger_service = self._login (username, password)
        except service.CaptchaRequired as message:
            hig_alert.reportError (error_name, _("A captcha was required for authentication.")) # TODO; propagate it
            success = False
        except service.BadAuthentication as message:
            hig_alert.reportError (error_name, _("Username or password was invalid."))
            success = False
        except service.Error as message:
            hig_alert.reportError (error_name, _("Failed to login: %s" % message))
            success = False
        except socket.gaierror as message:
            hig_alert.reportError (error_name, _("Socket error: %s.  Perhaps try again." % message));
            success = False

        if success == True:
            try:
                blogger_service.Post (post, "/feeds/%s/posts/default" % blog_id)
            except service.RequestError as message:
                hig_alert.reportError (error_name, _("Failed to post: %s" % message))

        return success

    def _loginPicasa(self, username, password):
        success = True
        error_name = "Failed Connection to Picasa"
        
        try:
            blogger_service = gdata.photos.service.PhotosService()
            blogger_service.ClientLogin(username, password)
        except service.CaptchaRequired as message:
            hig_alert.reportError (error_name, _("A captcha was required for authentication.")) # TODO; propagate it
            success = False
        except service.BadAuthentication as message:
            hig_alert.reportError (error_name, _("Username or password was invalid."))
            success = False
        except service.Error as message:
            hig_alert.reportError (error_name, _("Failed to login: %s" % message))
            success = False
        except socket.gaierror as message:
            hig_alert.reportError (error_name, _("Socket error: %s.  Perhaps try again." % message));
            success = False

        if success: 
            return blogger_service
        return None
 
    def _getDropBoxID(self, service, name):
        albums = service.GetUserFeed()
        for a in albums.entry:
            # print a.title.text
            # print a.gphoto_id.text
            if a.title.text == name:
                return a.gphoto_id.text
        return 0

    def _getThumb(self, photo, size, y_size):
        biggest = 0
        y_biggest = 0
        t = None
        
        for thumb in photo.media.thumbnail:
            if int(thumb.width) < size and int(thumb.width) > biggest and \
                    int(thumb.height) < y_size and int(thumb.height) > y_biggest:
                biggest = int(thumb.width)
                y_biggest = int(thumb.height)
                t = thumb
                
        if t == None:
            return photo.media.thumbnail[0].url
        return t.url
            
    def uploadImage (self, username, password, base_url, image, client, gconf_prefix):

        MAX_SIZE = 450 #It would be better if this got defined somewhere, perhaps the same place as window width ?
        MAX_SIZE_HEIGHT = 450

        #this part could be outsourced to an external lib if more APIS is going to use it
        file_path = image.uri[7:] 
        file_path = re.sub("%20", " ", file_path)
        file_name = image.name

        if (os.path.isfile(file_path)): # Test that we actually manage to find the file
            try:
                blogger_service = self._loginPicasa(username, password)
                albumid = self._getDropBoxID(blogger_service, "Drop Box")
                if albumid != 0:
                    album_url = '/data/feed/api/user/%s/albumid/%s' % (username, albumid)
                else:
                    hig_alert.reportError("Send Image", _("Drop Box folder was not found on Picasa"))
                    return ""
                photo = blogger_service.InsertPhotoSimple(album_url, file_name, \
                                          file_name, file_path, content_type=image.mime_type) #Send the file
                
                if int(photo.width.text) > MAX_SIZE or int(photo.height.text) > MAX_SIZE_HEIGHT: 
                    return "<a href=\"%s\" alt=\"%s\"><img src=\"%s\" /></a>" % \
                        (photo.content.src, file_name, self._getThumb(photo, MAX_SIZE, MAX_SIZE_HEIGHT))
                return "<img src=\"%s\" />" % (photo.content.src)
            except Exception, e:
                print e
                
        hig_alert.reportError ("Filename Error", _("Your filepath (%s) was invalid at this time." % file_path))
        print file_path
        return ""

blog = Blog()
